/* convert24to8	convert a HDF RGB 24 bit "pixel" image date into a 8 bit image
 *               with RGB palette 
 * orignal:    hdf24to8.c,v 1.4.4.1 1993/10/26 19:27:24 georgev Exp (NCSA)
 * modified :  xian
 */

#include <stdio.h>

/* Package-wide include files */

#include "reformat.h"
#include "extern.h"
#include "types.h"
#include "error.h"

/* HDF include files  */
#include "hdf.h"
#include "hfile.h"
#include "herr.h"
#include "df.h"

typedef	unsigned char	UCHAR;
typedef unsigned int	UINT;


int convert24to8(xres, yres, dat24, dat8, cres, cdat)
int32    xres;      /* x dimension - horizontal size */
int32    yres;      /* y dimension - vertical size */
UCHAR   *dat24;		/* pointer to 24 bit image in "pixel" format */
UCHAR   *dat8;		/* pointer to 8 bit image */
int      cres;		/* number of colors in the palette - use 256 */
UCHAR   *cdat;		/* pointer to palette - should be 3 * 256 bytes long */
 
{
    int      ct,xct,yct;
    int      rres,rd,rr,rn,rct;
    int      gres,gd,gr,gn,gct;
    int      bres,bd,br,bn,bct;
    int      coff;
    UINT    *idat[2];
    UINT    *cp,*np;
    UCHAR   *dip,*dop,*rp,*gp,*bp;
#ifdef UNIX
    char    *malloc();
#endif

    if ((idat[0] = (UINT *)HDgetspace(6*xres*sizeof(UINT))) == NULL)
    {   fprintf(stderr,"error: Memory allocation fault\n");
	return -1;
    }
    idat[1] = idat[0] + (3 * xres);

    rres = 6;
    gres = 7;
    bres = 6;
    coff = 2;

    rr = gr = br = 255;
    rn = rres - 1;
    gn = gres - 1;
    bn = bres - 1;

    rp = cdat + coff;
    gp = rp + cres;
    bp = gp + cres;

    for (rct=0; rct<rres; rct++)
    {   for (gct=0; gct<gres; gct++)
	{   for (bct=0; bct<bres; bct++)
	    {   *rp++ = (UCHAR)(rr * rct / rn);
		*gp++ = (UCHAR)(gr * gct / gn);
		*bp++ = (UCHAR)(br * bct / bn);
	    }
	}
    }

    rp = cdat;
    gp = rp + cres;
    bp = gp + cres;
    cp = idat[0];
    np = idat[1];
    dip = dat24;
    dop = dat8;

    for (xct=(intn)(3*xres); --xct>=0; )
        *cp++ = *dip++;

    for (yct=0; yct<(yres-1); yct++)
    {
	np = idat[(yct+1)%2];
    for (xct=(intn)(3*xres); --xct>=0; )
	    *np++ = *dip++;

	cp = idat[yct%2];
	np = idat[(yct+1)%2];

	if ((rct = (cp[0] * rn / rr)) > rn) rct = rn;
	if ((gct = (cp[1] * gn / gr)) > gn) gct = gn;
	if ((bct = (cp[2] * bn / br)) > bn) bct = bn;

    *dop++ = (UCHAR)(ct = (rct * gres + gct) * bres + bct + coff);

	rd = cp[0] - rp[ct];
	gd = cp[1] - gp[ct];
	bd = cp[2] - bp[ct];

	cp += 3;
	np += 3;

	cp[0]  += rd * 7 / 16;
	cp[1]  += gd * 7 / 16;
	cp[2]  += bd * 7 / 16;
	np[-3] += rd * 5 / 16;
	np[-2] += gd * 5 / 16;
	np[-1] += bd * 5 / 16;
	np[0]  += rd / 16;
	np[1]  += gd / 16;
	np[2]  += bd / 16;

	for (xct=2; xct<xres; xct++)
	{
	    if ((rct = (cp[0] * rn / rr)) > rn) rct = rn;
	    if ((gct = (cp[1] * gn / gr)) > gn) gct = gn;
	    if ((bct = (cp[2] * bn / br)) > bn) bct = bn;

        *dop++ = (UCHAR)(ct = (rct * gres + gct) * bres + bct + coff);

	    rd = cp[0] - rp[ct];
	    gd = cp[1] - gp[ct];
	    bd = cp[2] - bp[ct];

	    cp += 3;
	    np += 3;

	    cp[0]  += rd * 7 / 16;
	    cp[1]  += gd * 7 / 16;
	    cp[2]  += bd * 7 / 16;
	    np[-6] += rd * 3 / 16;
	    np[-5] += gd * 3 / 16;
	    np[-4] += bd * 3 / 16;
	    np[-3] += rd * 5 / 16;
	    np[-2] += gd * 5 / 16;
	    np[-1] += bd * 5 / 16;
	    np[0]  += rd / 16;
	    np[1]  += gd / 16;
	    np[2]  += bd / 16;

	}

	if ((rct = (cp[0] * rn / rr)) > rn) rct = rn;
	if ((gct = (cp[1] * gn / gr)) > gn) gct = gn;
	if ((bct = (cp[2] * bn / br)) > bn) bct = bn;

    *dop++ = (UCHAR)(ct = (rct * gres + gct) * bres + bct + coff);

	rd = cp[0] - rp[ct];
	gd = cp[1] - gp[ct];
	bd = cp[2] - bp[ct];

	cp += 3;
	np += 3;

	np[-6] += rd * 3 / 16;
	np[-5] += gd * 3 / 16;
	np[-4] += bd * 3 / 16;
	np[-3] += rd * 5 / 16;
	np[-2] += gd * 5 / 16;
	np[-1] += bd * 5 / 16;
    }

    cp = idat[yct%2];

    if ((rct = (cp[0] * rn / rr)) > rn) rct = rn;
    if ((gct = (cp[1] * gn / gr)) > gn) gct = gn;
    if ((bct = (cp[2] * bn / br)) > bn) bct = bn;

    *dop++ = (UCHAR)(ct = (rct * gres + gct) * bres + bct + coff);

    rd = cp[0] - rp[ct];
    gd = cp[1] - gp[ct];
    bd = cp[2] - bp[ct];

    cp += 3;

    cp[0]  += rd * 7 / 16;
    cp[1]  += gd * 7 / 16;
    cp[2]  += bd * 7 / 16;

    for (xct=2; xct<xres; xct++)
    {
	if ((rct = (cp[0] * rn / rr)) > rn) rct = rn;
	if ((gct = (cp[1] * gn / gr)) > gn) gct = gn;
	if ((bct = (cp[2] * bn / br)) > bn) bct = bn;

    *dop++ = (UCHAR)(ct = (rct * gres + gct) * bres + bct + coff);

	rd = cp[0] - rp[ct];
	gd = cp[1] - gp[ct];
	bd = cp[2] - bp[ct];

	cp += 3;

	cp[0]  += rd * 7 / 16;
	cp[1]  += gd * 7 / 16;
	cp[2]  += bd * 7 / 16;
    }

    if ((rct = (cp[0] * rn / rr)) > rn) rct = rn;
    if ((gct = (cp[1] * gn / gr)) > gn) gct = gn;
    if ((bct = (cp[2] * bn / br)) > bn) bct = bn;

    *dop++ = (UCHAR)((rct * gres + gct) * bres + bct + coff);

    free(idat[0]);
    return 0;
}



/* finding max & min values from datasets */
#define findMaxMin(x, dataMax, dataMin) {\
tmp=(x);  if(tmp > dataMax) dataMax=tmp; \
if (tmp<dataMin) dataMin=tmp;}


/* convert dataSet to image */

ErrorCode convert2image(fname, info)
    char     *fname;
    FileInfo *info;
{ 
    uint8 *image=info->image;
    int i;
    float32 tmp, *buffer1,*buffer2;
    float32 scale, spread ;
    float32 dataMax,dataMin;
    
    buffer1 = buffer2 = (float *)info->data;
/*  if HDF science data have not max and min value don't use DFSDgetrange */
/*
    dataMax = 9.99e+20; 
    dataMin = -9.99e+20;

    if (!DFSDgetrange((VOIDP)&dataMax, (VOIDP)&dataMin))
	{ info->max = (float) dataMax;
	info->min = (float) dataMin;
	}
    else
*/
    dataMax = dataMin =(float32)info->data[0];
    for (i=0; i<info->width*info->height;buffer1++, i++)
	findMaxMin(*buffer1, dataMax, dataMin);
    
    spread = (dataMax == dataMin) ? 1. : 255./((float32) dataMax -(float32) dataMin);
    for (i=0; i<info->width*info->height; buffer2++,i++,image++)
	{*image = (uint8) (((float)*buffer2 - dataMin)*spread);
	}


return(AllOk);
}




