/* hdf2ras.c
 * Convert  an HDF image to a sun raster file.
 *
 */

#include <stdio.h>

/* Package-wide include files */

#include "reformat.h"
#include "extern.h"
#include "types.h"
#include "error.h"

/* HDF include files  */
#include "hdf.h"
#include "hfile.h"
#include "herr.h"
#include "df.h"


ErrorCode Hdf2Ras(fname, info)
    char *fname;
    FileInfo *info;
{
    int32  fid, aid, data_aid;
    int    i,j, imageNumber, isPal,interlace, err_val, len;
    int32  xdim, ydim;
    uint8 *image , palette[MAX_PAL], *image24;
    char  scratch[128],scratch1[60],scratch2[20];      /* name of the raw file to create */

    int32  dimSize[10],num_type;
    int    rank,dataSpace,ret;
    uint8  *data;
 
    len = at("#",info->name);
    if (len>0) {
    strcpy(scratch1,substr(info->name,1,len));
    strcpy(scratch2,substr(info->name,len+2,strlen(info->name)+2));
    }
    else len = 0;

    /* Check for the HDF file and get info. to fill the info. */

    fid = Hopen(fname, DFACC_READ, -1);
    if (fid == FAIL) 
	return(err_msg(Hdf2Ras, InputOpenFailed));
    DFR8restart();
    for (imageNumber =1 ; !DFR8getdims(fname, &xdim,&ydim, &isPal);imageNumber++){      
	image = (uint8*)newSpace(xdim * ydim);
	if (!DFR8getimage(fname, image, xdim, ydim, palette)){
	    info->image = image;
	    info->width = xdim;
	    info->height= ydim;
	    if (isPal){
	      /*	      strcpy(info->palette ,palette);*/
		for (i=0; i<MAX_PAL; i++)
		    info->palette[i] = palette[i];
		info->values |= CvtPalette;
		info->values |= CvtImage;
	    }
	    else{
		for (i=0; i<= 255; i++){
		    info->palette[3*i  ] = 255 -i;
		    info->palette[3*i+1] = 255 -i;
		    info->palette[3*i+2] = 255 -i;
		}
		info->values |= CvtPalette;
		info->values |= CvtImage;
	    }
	    /* modified the info->name, if it include symbol '#' the '#'
	     * will be replaced with imageNumer
	     */
	    if (len){
		strcpy(scratch, scratch1);
		strcat(scratch, num2str(imageNumber));
		strcat(scratch, scratch2);
		strcpy(info->name, scratch);
	    }
	}  /* (!HDF8getimage(fname, image, xdim, ydim, palette)) */
	err_val = HEvalue(1);
	if ((err_val != DFE_NOMATCH) && (err_val != DFE_NONE)) {
	    HEprint(stderr,0);
	    exit(1);
	}
	/* write image to raw file */
	WriteSunrast(info);
    }
    
    /* if HDF file include 24 bits image convert it to 8 bits */
    DF24restart();
    for (j=imageNumber; !DF24getdims(fname, &xdim, &ydim, &interlace); j++){      
	image24 = (uint8*)newSpace(xdim * ydim * 3);
	image   = (uint8*)newSpace(xdim * ydim);
	if (!DF24getimage(fname, image24, xdim, ydim)){
	    info->width = xdim;
	    info->height= ydim;
	    /*  convert 24 bits image datas to 8 bits */
	    convert24to8(xdim, ydim, image24, image,256, palette);
	    info->image = image;
	    for (i=0; i<MAX_PAL/3; i++){
		info->palette[3*i]   = palette[i];
		info->palette[3*i+1] = palette[i+256];
		info->palette[3*i+2] = palette[i+256*2];
	    }
	    info->values |= CvtPalette;
	    info->values |= CvtImage;
	    /* modified the info->name, if it include symbol '#' the '#'
	     * will be replaced with imageNumer
	     */
	    if (len){
		strcpy(scratch, scratch1);
		strcat(scratch, num2str(j));
		strcat(scratch, scratch2);
		strcpy(info->name, scratch);
	    }
	}  /* (!HDF8getimage(fname, image, xdim, ydim, palette)) */
	err_val = HEvalue(1);
	if ((err_val != DFE_NOMATCH) && (err_val != DFE_NONE)) {
	    HEprint(stderr,0);
	    exit(1);
	}
	/* write image to raw file */
	WriteSunrast(info);
    }
    
    /* get data info */
    DFSDrestart();
    for (i=imageNumber; !DFSDgetdims(fname, &rank,dimSize,10); i++){
	float *p;
	if ((rank == 2))  {
	    /* return(err_msg(Hdf2Ras, BadInputFile)); */

	DFSDgetNT(&num_type);
        dataSpace = dimSize[0];
	for (j=1; j<rank; j++)
	    dataSpace *= dimSize[j];
	p = info->data = (float *)malloc(dimSize[0] * dimSize[1] * sizeof(float));
	if (p == (float *)NULL)
	    return(err_msg(Hdf2Ras, NoMemory));

	switch(num_type) {
	case DFNT_UINT8: {
	    uint8 *outDataUint8;
	    dataSpace *= sizeof(uint8);
	    outDataUint8 = (uint8 *)malloc(dataSpace);
	    ret = DFSDgetdata(fname, rank, dimSize,(VOIDP)outDataUint8);
	    if (!ret) {
		for (j=0; j< dimSize[0]*dimSize[1]; j++, p++, outDataUint8++)
		    *p = (float) *outDataUint8;
		free(outDataUint8); 
	    }
	    break;
	} 

	case DFNT_FLOAT32: {
	    /* 32-bit floats */
	    float32 *outData32;
	    dataSpace *= sizeof(float32);
	    outData32 = (float32 *)malloc(dataSpace);
	    if (outData32 == (float32 *)NULL)
		return(err_msg(Hdf2Ras, NoMemory));

	    ret = DFSDgetdata(fname, rank, dimSize,(VOIDP)outData32);
	    if (!ret) {
		for (j=0; j<(dimSize[0] * dimSize[1]); j++, p++, outData32++)
		    *p = (float) *outData32;
		free(outData32);   
	    } 
	    break;
	}
	case  DFNT_FLOAT64: {
	    /* 64-bit floats  */
	    float64 *outData64;
	    dataSpace *= sizeof(float64);
	    outData64 = (float64 *)malloc(dataSpace);
	    ret = DFSDgetdata(fname, rank, dimSize,(VOIDP)outData64);
	    if (!ret) {
		for (j=0; j< dimSize[0] * dimSize[1]; j++ , p++, outData64++)
		    *p = (float) *outData64;
		free(outData64);   
	    }
	    break;

	}
	case DFNT_INT16: {	   
	    int16 *outData16;
	    dataSpace *= sizeof(int16);
	    outData16 = (int16 *)malloc(dataSpace);
	    ret = DFSDgetdata(fname, rank, dimSize,(VOIDP)outData16);
	    if (!ret) 
		for (j=0; j< dimSize[0] * dimSize[1]; j++, p++, outData16++)
		    *p = (float) *outData16;
	    free(outData16);   
	    break;
	}
  	case DFNT_UINT16: {	   
	    uint16 *outDataU16;
	    dataSpace *= sizeof(uint16);
	    outDataU16 = (uint16 *)malloc(dataSpace);
	    ret = DFSDgetdata(fname, rank, dimSize,(VOIDP)outDataU16);
	    if (!ret) {
		for (j=0; j< dimSize[0] * dimSize[1]; j++, p++, outDataU16++)
		    *p = (float) *outDataU16;
		free(outDataU16);       
	    }
	    break;
	} 
	case DFNT_INT32: {	   
	    int32 *outDataInt32;
	    dataSpace *= sizeof(int32);
	    outDataInt32 = (int32 *)malloc(dataSpace);
	    ret = DFSDgetdata(fname, rank, dimSize,(VOIDP)outDataInt32);
	    if (!ret) {
		for (j=0; j< dimSize[0] * dimSize[1]; j++, p++, outDataInt32++)
		    *p = (float) *outDataInt32;
		free(outDataInt32);    
	    }
	    break;
	} 
	case DFNT_UINT32: {
	    uint32 *outDataUint32;
	    dataSpace *= sizeof(uint32);
	    outDataUint32 = (uint32 *)malloc(dataSpace);
	    ret = DFSDgetdata(fname, rank, dimSize,(VOIDP)outDataUint32);
	    if (!ret) {
		for (j=0; j< dimSize[0]*dimSize[1]; j++, p++, outDataUint32++)
		    *p = (float) *outDataUint32;
		free(outDataUint32); 
	    }
	    break;
	} 
	default:
	    return(err_msg(Hdf2Ras, FileSystemError));
	}

	if (!ret){	 

	    info->ranks = (long *)malloc(rank * sizeof(long));
	    for (j=0; j<rank; j++)
		info->ranks[j] = dimSize[j];
	    info->dimension = rank;
	    info->values = 0;
	    info->values |= CvtData;
	    info->width = dimSize[1];
	    info->height= dimSize[0];
	    /* making a nice palette */
	    for (j=0; j< MAX_PAL/3; j++){
		info->palette[3*j  ] = j;
		info->palette[3*j+1] = j;
		info->palette[3*j+2] = j;
	    }
	    info->image = (uint8 *)malloc(info->width * info->height);
	    if (info->image == (uint8 *) NULL)
		return(err_msg(Hdf2Ras, NoMemory));
  	    
	    convert2image(fname,info);
 
/* modified the info->name, if it include symbol '#' the '#'
 * will be replaced with imageNumer
 */
	    if (len){
	      strcpy(scratch, scratch1);
	      strcat(scratch, num2str(i));
	      strcat(scratch, scratch2);
	      strcpy(info->name, scratch);
	    }
	} /* (!HDF8getimage(fname, image, xdim, ydim, palette)) */
	err_val = HEvalue(1);
	if ((err_val != DFE_NOMATCH) && (err_val != DFE_NONE)) {
	    HEprint(stderr,0);
	    exit(1);
	}
	/* write image to raw file */
	WriteSunrast(info);
	}
    }
    return(Done);   
}







