/* write_gif.c
 * Write data out to a GIF file.
 *  
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <stdlib.h>
/* Package include files */

#include "reformat.h"
#include "types.h"
#include "extern.h"
#include "error.h"

typedef char              int8;
typedef unsigned char     uint8;
typedef short int         int16;
typedef unsigned short int uint16;
typedef long              int32;
typedef unsigned long int uint32;

#define IMAGESEP 0x2c;      /* ,   */
#define IMAGEEND 0x3b;      /* ;   */
#define INTERLACEMASK 0x40;
#define COLORMAPMASK 0x80;
#define GIF_MAGIC 0x474946383761;    /* 0x474946383761( GIF87a)    */

typedef struct imageDesc {
    uint8    imageSeparator;
    int16  imageLeft, imageTop;
    int16  imageWidth, imageHeight;
    uint8    localColorMapInfo;
} imageDescription;

typedef struct gifHeader {
    uint8  gifTag[6]; 
    int16  screenWidth, screenHeight;
    uint8  globalColorMapInfo,  bgColorTag, colorDepthInfo;   
 
} gifHeaderInfo;

/* Write out a GIF file */
ErrorCode WriteGIF(info)
    FileInfo *info;
{
    FILE   *fp;         /* gif file identifer   */
    int16 sWidth, sHeight, iWidth, iHeight, iTop, iLeft;
    int background;
    int colorNumber;   
    int hasColorMap, interlace;
    gifHeaderInfo    *tmpGifHeaderInfo;
    uint8 *image;
    uint8 *rawGIFImage;
    uint8 red[256], green[256], blue[256];
    imageDescription *tmpImageDescription;
    int imageEnd, i;
    if (info == NULL) return(err_msg(WriteGIFFile, NullPtr));
    for(i=0; i<MAX_PAL/3; i++){
	red[i]     = info->palette[3*i];
	green[i] = info->palette[3*i+1];
	blue[i]  = info->palette[3*i+2];
    }
/* Open the file */
    fp = fopen(info->name,"w");
    if (fp == NULL) 
	return(err_msg(WriteGIFFile, OutputOpenFailed));
/* picture = 8 bits images   */
    hdfWriteGIF(fp, info->image, 0, info->width, info->height, red, green, blue, 256,8, "");
    fclose(fp);
    return(AllOk);
}












