/* write_tiff.c
 * Write data out to a TIFF file.
 *
 */
#include <stdio.h>
#include <sys/time.h>

/* tiff include file   */
#include "tiffio.h"

/* Package include files */

#include "reformat.h"
#include "types.h"
#include "error.h"
#include "extern.h"
typedef char int8;
#define MAX(a,b) (((a) > (b)) ? (a) : (b))

/* Write out a TIFF file */

ErrorCode WriteTIFF(info)
    FileInfo *info;
{
    FILE *fp;
    char *inp=NULL;
    char *outp=NULL;
    int depth,i,row,col;
    TIFF *tif;
    uint16 red[256],green[256],blue[256];
    struct tm *ct;
    struct timeval tv;
    int32 width,height;
    int32 rowsperstrip;
    int16 photometric;
    int16 samplesperpixel;
    int16 bitspersample;
    int bpsl;
    static char *version = "WriteTIFF 1.0";
    static char datetime[] = "1995:03:01 11:18:00";
    
    if (info == NULL) return(err_msg(WriteTIFFFile, NullPtr));
    gettimeofday(&tv,(struct timezone *) NULL);
    ct = localtime((time_t *)&tv.tv_sec);
    sprintf(datetime,"19%02d:%02d:%02d %02d:%02d:%02d", ct->tm_year, ct->tm_mon+1, ct->tm_mday, ct->tm_hour, ct->tm_min, ct->tm_sec);

/* Open the file */
    tif = TIFFOpen(info->name,"w");
    if (tif == NULL) return(err_msg(WriteTIFFFile, OutputOpenFailed));
    
    width = info->width;
    height = info->height;
    depth = 8;                      /* assume to write 8 bits palette RGB image   */
    switch(depth){
    case 1:           /* white & black image  */
	samplesperpixel = 1;
	bitspersample = 1;
	photometric = PHOTOMETRIC_MINISBLACK;
	break;
    case 8:      /* 8 bits palette RGB  */
	samplesperpixel = 1;
	bitspersample = 8;
	photometric = PHOTOMETRIC_PALETTE;
	break;
    default:
	return(err_msg(WriteTIFFFile, FileSystemError));
    }

    rowsperstrip = MAX(1, 8192/width);

    TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, width);
    TIFFSetField(tif, TIFFTAG_IMAGELENGTH, height);
    TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE,bitspersample);
    TIFFSetField(tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
    if (CompressOBJ)
	TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_LZW);
    else
	TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
    TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, photometric);
    TIFFSetField(tif, TIFFTAG_DOCUMENTNAME, info->name);
    TIFFSetField(tif, TIFFTAG_IMAGEDESCRIPTION, "Convert HDF file to Tiff image");
    TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, samplesperpixel);
    TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP,rowsperstrip);      
    TIFFSetField(tif, TIFFTAG_PLANARCONFIG,PLANARCONFIG_CONTIG);
    TIFFSetField(tif, TIFFTAG_SOFTWARE , version );                  
    TIFFSetField(tif, TIFFTAG_DATETIME, datetime);
    if(depth == 8){
	for (i=0; i < MAX_PAL/3; i++) {
	    red[i]   = info->palette[3*i  ]*256+127;
	    green[i] = info->palette[3*i+1]*256+127;
	    blue[i]  = info->palette[3*i+2]*256+127;
	}
	TIFFSetField(tif, TIFFTAG_COLORMAP,red,green,blue); 
    } 
/*    TIFFWriteEncodedStrip(tif,0,info->image,width*height);*/
    for (row = 0; row < height; row++)
	TIFFWriteScanline(tif, info->image+(row*width), row, 0);
    TIFFFlushData(tif);

    TIFFClose(tif);
  return(Done);
}


