/* write_xwd.c
 * Write data out to a XWD file.
 *
 */

#include <stdio.h>
#include <string.h>
#include "x11wd.h"

/* Package include files */
#include "reformat.h"
#include "types.h"
#include "extern.h"
#include "error.h"

/* X11 include files */
#include <X11/Xlib.h>

typedef unsigned long dw;
typedef unsigned int  word;
#define MAX_CELLS 256

typedef struct {
  XImage *ximage;
  word numcells;
  word red[MAX_CELLS], green[MAX_CELLS], blue[MAX_CELLS];
  byte used[MAX_CELLS];
} imageInfo;

/* Write out a XWD file */
ErrorCode WriteXWD(info)
    FileInfo *info;
{
  Display *hDisplay;
  int      hScreen;
  char imageName[62];
  FILE *fp;
  char *ptr,*img;
  imageInfo *image;
  XImage  *ximage;        /* image->ximage = ximage   */
  int i;
  if (info == NULL) return(err_msg(WriteXWDFile, NullPtr));
  /* open the output stream */
  if (info->name) {
    fp = fopen(info->name, "w");
    if (!fp) 
	return(err_msg(WriteXWDFile, FileSystemError));
    /* form an image name based on the file name */
    strcpy(imageName, info->name);
  }
  else
      strcpy(imageName,"empty");  
/* initialize ximage  */
  if ((image = (imageInfo *) malloc(sizeof(imageInfo))) == NULL)
      return(err_msg(WriteXWDFile,NoMemory));
  if ((ximage = (XImage *) malloc(sizeof(XImage))) == NULL)
      return(err_msg(WriteXWDFile,NoMemory));

  ximage->width  = info->width;
  ximage->height = info->height;
  ximage->xoffset = 0;
  ximage->format = ZPixmap;
  ximage->byte_order = MSBFirst;
  ximage->bitmap_unit = 8;
  ximage->bitmap_bit_order=MSBFirst;
  ximage->bitmap_pad = 32;      /* ZPixmap   */
  ximage->depth = 8;       /* depth of image     */
  ximage->bytes_per_line = info->width ;
  ximage->bits_per_pixel = 8;   /* bits per pixel (ZPixmap */
  
  if ((ximage->data = (char *) malloc(ximage->width * ximage->height)) == NULL)
      return(err_msg(WriteXWDFile,NoMemory));

  ximage->data = (char *)info->image;
  /* initialize image  */
  image->numcells = MAX_CELLS;
  for (i=0; i<MAX_CELLS; i++){
      image->red[i]  =info->palette[3*i  ]*65535/255;
      image->green[i]=info->palette[3*i+1]*65535/255;
      image->blue[i] =info->palette[3*i+2]*65535/255;
  }
  image->ximage = ximage;
  x11writeXWD(image,fp ,1,imageName);
  free(image);
  free(ximage);
  fclose(fp);
  return(Done);
}


x11writeXWD(image, outfile, xyformat,imageName)
  imageInfo *image;
  FILE *outfile;
  int xyformat;
  char imageName[62];
{
  XImage   *ximage = image->ximage;
  X11WDFileHeader header;
  Visual   *visual;
  XColor    color;
  dw        visMask; 
  dw        swaptest = 1;
  int       i, x, y, size;
  char      *ptr;
  int	    XYtoZ;
  byte	    b;
  int	    bc;
  char      *display;
  int       hScreen;
  Display   *hDisplay;

  display  = NULL;
  hDisplay = XOpenDisplay(display);
  if (!hDisplay) {
    fprintf(stderr, "Could not open X display\n");
    exit(3);
  }
  hScreen  = DefaultScreen(hDisplay);
  visual = DefaultVisual(hDisplay, hScreen);  
  visMask= (visual->red_mask
                      | visual->green_mask
                      | visual->blue_mask);
  header.header_size    = (CARD32)(sizeof(header)+strlen(imageName)+1);
  header.file_version   = (CARD32) X11WD_FILE_VERSION;
  XYtoZ = (ximage->depth==1 && !xyformat);
  header.pixmap_format  = (CARD32)(ximage->depth>1? ZPixmap : (xyformat? XYPixmap : ZPixmap));
  header.pixmap_depth   = (CARD32) (XYtoZ? 8 : ximage->depth);
  header.pixmap_width   = (CARD32) ximage->width;
  header.pixmap_height  = (CARD32) ximage->height;
  header.xoffset        = (CARD32) ximage->xoffset;
  header.byte_order     = (CARD32) ximage->byte_order;
  header.bitmap_unit    = (CARD32) (XYtoZ ? 8 : ximage->bitmap_unit);
  header.bitmap_bit_order = (CARD32) (XYtoZ ? MSBFirst : ximage->bitmap_bit_order);
  size = XYtoZ ? ximage->bitmap_pad * 8 : ximage->bitmap_pad;
  header.bitmap_pad     = (CARD32) size;
  header.bits_per_pixel = (CARD32) XYtoZ? 8 : ximage->bits_per_pixel;
  size = XYtoZ? ximage->width : ximage->bytes_per_line;
  header.bytes_per_line = (CARD32)size;
  header.visual_class   = (CARD32)visual->class;
  header.red_mask       = (CARD32)visual->red_mask;
  header.green_mask     = (CARD32)visual->green_mask;
  header.blue_mask      = (CARD32)visual->blue_mask;
  header.bits_per_rgb   = (CARD32)visual->bits_per_rgb;
  header.colormap_entries = (CARD32)visual->map_entries;
  /* ncolors should be image->numcells, but some programs seem to expect
   * that it is == colormap_entries.  sigh */
  header.ncolors        = header.colormap_entries;
  header.window_width   = (CARD32)ximage->width;
  header.window_height  = (CARD32)ximage->height;
  header.window_x       = 0;
  header.window_y       = 0;
  header.window_bdrwidth = 0;

  if (*(char *) &swaptest)
    swapdws(&header, sizeof(header));

  fwrite(&header, sizeof(header), 1, outfile);
  fwrite(imageName, 1, strlen(imageName)+1, outfile);

  if (*(char *) &swaptest)
    swapdws(&header, sizeof(header));

  for (i=0; i<image->numcells; i++) {
    color.pixel = i;
    color.red   = image->red[i];
    color.green = image->green[i];
    color.blue  = image->blue[i];
    color.flags = visMask;
    color.pad   = 0;
    if (*(char *) &swaptest) {
      swapdws(&color.pixel, sizeof(color.pixel));
      swapwords(&color.red, 3 * sizeof(color.red)); /* assume g and b follow r */
    }
    fwrite(&color, sizeof(XColor), 1, outfile);
  }

  /* pad out with black entries */
  color.red = color.green = color.blue = 0;
  color.flags = visMask;
  color.pad = 0;
  for (i=image->numcells; i<visual->map_entries; i++) {
    color.pixel = i;
    if (*(char *) &swaptest)
      swapdws(&color.pixel, sizeof(color.pixel));
    fwrite(&color, sizeof(XColor), 1, outfile);
  }
  
  if (XYtoZ) {
    for (y = 0; y < ximage->height; y++) {
      for (x = 0; x < ximage->width; x++) {
        b = XGetPixel(ximage, x, y);
	fputc(b, outfile);
      }
    }
  }
  else {
    if (ximage->depth == 1  &&  BlackPixel(hDisplay,hScreen) == 0) {
      size = ximage->height * ximage->bytes_per_line;
      for (i=0, ptr=&ximage->data[0]; i<size; i++, ptr++)
        fputc(~(*ptr), outfile);
    }
    else
      fwrite(ximage->data, ximage->height * ximage->bytes_per_line, 1, outfile);
  }

}














/* swap the bits in a byte */
swapbits(b)
  byte b;
{
  byte b2;

  b2 = 0;
  b2 |= (b & 0x01) << 7;
  b2 |= (b & 0x02) << 5;
  b2 |= (b & 0x04) << 3;
  b2 |= (b & 0x08) << 1;
  b2 |= (b & 0x10) >> 1;
  b2 |= (b & 0x20) >> 3;
  b2 |= (b & 0x40) >> 5;
  b2 |= (b & 0x80) >> 7;
  return b2;
}




/* swap the bytes in a long int */
swapbytes(pDblw)
  dw *pDblw;
  {
  union {
    dw  dbl;
    byte bytes[4];
    } cnv;
  byte aByte;

  cnv.dbl = *pDblw;
  aByte = cnv.bytes[0];
  cnv.bytes[0] = cnv.bytes[3];
  cnv.bytes[3] = aByte;
  aByte = cnv.bytes[1];
  cnv.bytes[1] = cnv.bytes[2];
  cnv.bytes[2] = aByte;
  *pDblw = cnv.dbl;
  }



/* swap some long ints.  (n is number of BYTES, not number of longs) */
swapdws (bp, n)
  register char *bp;
  register unsigned n;
{
  register char c;
  register char *ep = bp + n;
  register char *sp;

  while (bp < ep) {
    sp = bp + 3;
    c = *sp;
    *sp = *bp;
    *bp++ = c;
    sp = bp + 1;
    c = *sp;
    *sp = *bp;
    *bp++ = c;
    bp += 2;
  }
}



/* swap some short ints */
swapwords (bp, n)
  register char *bp;
  register unsigned n;
{
  register char c;
  register char *ep = bp + n;

  while (bp < ep) {
    c = *bp;
    *bp = *(bp + 1);
    bp++;
    *bp++ = c;
  }
}



